<xsl:stylesheet version = '1.0' 
		xmlns:xsl='http://www.w3.org/1999/XSL/Transform' 
		xmlns:msxsl="urn:schemas-microsoft-com:xslt"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns:cvs="http://releng.wtp.eclipse.org/cvstools"
		xsi:schemaLocation="./cvstools.xsd
         http://www.w3.org/1999/XSL/Transform
         http://www.w3.org/1999/XSL/Transform.xsd
         http://www.w3.org/1999/xhtml
         http://www.w3.org/1999/xhtml.xsd">
	<xsl:output method="html" encoding="ISO-8859-1"/>

	<!-- set default values here, and also in php script wrapper-->
	<xsl:param name="iesDate"></xsl:param>
	<xsl:param name="drvDate"></xsl:param>
	<xsl:param name="cvsServer">http://dev.eclipse.org</xsl:param>
	<xsl:param name="source"/>

	<!-- points to a specific bug's logfile; not the same as Bugzilla, which is filterName4 -->
	<xsl:param name="bug"/>

	<xsl:param name="sortMethod1">date</xsl:param> 
	<xsl:param name="sortDir1">descending</xsl:param> 
	<xsl:param name="sortMethod2">file</xsl:param> 
	<xsl:param name="sortDir2">ascending</xsl:param> 

	<!-- up to 5 different filters can be applied -->
	<xsl:param name="filterName1">File</xsl:param>
	<xsl:param name="filterVal1"></xsl:param>
	<xsl:param name="filterName2">Date</xsl:param>
	<xsl:param name="filterVal2"></xsl:param>
	<xsl:param name="filterName3">Author</xsl:param>
	<xsl:param name="filterVal3"></xsl:param>
	<xsl:param name="filterName4">Bugzilla</xsl:param>
	<xsl:param name="filterVal4"></xsl:param>
	<xsl:param name="filterName5">Comments</xsl:param>
	<xsl:param name="filterVal5"></xsl:param>

	<xsl:param name="CVSPREWTP">http://dev.eclipse.org/viewcvs/indexwebtools.cgi/~checkout~/org.eclipse.wtp.releng/cvstools/org.eclipse.wtp.releng.cvs.tools</xsl:param>

	<xsl:template match="/">
		<xsl:for-each select="cvs:rlog">
			<html>
			<head>
			<title>Eclipse Web Tools Platform: What`s New, CVS?</title>
			<style>@import url("../whatsnew-cvs.css");</style>
			</head>
			<body>
			<a name="top"> </a>
			<!-- iesDate: <xsl:value-of select="$iesDate" />, drvDate: <xsl:value-of select="$drvDate" /><br/> -->
			<b class="big-header">
				<xsl:choose>
					<xsl:when test="$drvDate=cvs:startdate and $iesDate=cvs:startdate">
						Between Last Uploaded Driver &amp; IES Refresh On <b class="navy"><xsl:value-of select="cvs:startdate" /></b> and <b class="navy"><xsl:value-of select="cvs:enddate" /></b>
					</xsl:when>
					<xsl:when test="$drvDate=cvs:startdate">
						Between Uploaded Driver On <b class="navy"><xsl:value-of select="cvs:startdate" /></b> and <b class="navy"><xsl:value-of select="cvs:enddate" /></b>
					</xsl:when>
					<xsl:when test="$iesDate=cvs:startdate">
						Between Last IES Refresh On <b class="navy"><xsl:value-of select="cvs:startdate" /></b> and <b class="navy"><xsl:value-of select="cvs:enddate" /></b>
					</xsl:when>
					<xsl:otherwise>
						Between 
						<b class="navy">
							<xsl:choose>
							<xsl:when test="contains(cvs:startdate,'00:00:00')">
								<xsl:value-of select="substring-before(cvs:startdate,'00:00:00')"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="cvs:startdate"/>
							</xsl:otherwise>
							</xsl:choose>
						</b> and 
						<b class="navy"><xsl:value-of select="cvs:enddate"/></b>
					</xsl:otherwise>
				</xsl:choose>
			</b>
			<h4 class="green">CVS Deltas: <xsl:value-of select="cvs:counts/cvs:numDeltas"/>, Revisions: <xsl:value-of select="cvs:counts/cvs:numRevisions"/></h4>

			<table width="98%" border="0">
				<tr>
					<td><b></b></td> <!-- # --> 
					<td><b>Group</b></td>
					<td align="right"><b>C</b></td> <!-- link to CVS parent folder -->
					<td align="center"><b>V</b></td> <!-- link to CVS script detail page -->
					<td align="left"><b>S</b></td> <!-- link to CVS source (~checkout~) page -->
					<td align="left"><b>File</b></td> <!-- link to CVS source (~checkout~) page -->
					<td align="left"><b>Versions</b></td> <!-- link to CVS source (~checkout~) page -->
					<td width="100%" colspan="2"><table  width="100%"><tr><td align="left"><b>Date</b></td> <!-- link to CVS source (~checkout~) page -->
					<td align="left"><b>Author</b></td> <!-- link to CVS source (~checkout~) page -->
					<td align="left"><b>Bugzilla</b></td> <!-- link to CVS source (~checkout~) page -->
					<td align="left"><b>Comment</b></td> <!-- link to CVS source (~checkout~) page -->
					</tr></table></td>
				</tr>
				<xsl:for-each select="cvs:cvsdelta">
					<xsl:sort select="cvs:revision/*[name()=$sortMethod1]" order="{$sortDir1}"/> 
					<xsl:sort select="./*[name()=$sortMethod2]" order="{$sortDir2}"/>

					<!-- add filters -->
					<xsl:variable name="doRow">
					<xsl:choose>
						<xsl:when test="$filterName1='File' and $filterVal1!=''">
							<xsl:if test="contains(.,$filterVal1)">1</xsl:if> <!-- found -->
						</xsl:when>
						<xsl:when test="$filterName2='Date' and $filterVal2!=''">
							<xsl:for-each select="cvs:revision">
								<xsl:choose>
									<xsl:when test="contains(cvs:date,$filterVal2)">1</xsl:when> <!-- found -->
									<xsl:otherwise></xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
						</xsl:when>
						<xsl:when test="$filterName3='Author' and $filterVal3!=''">
							<xsl:for-each select="cvs:revision">
								<xsl:choose>
									<xsl:when test="contains(cvs:author,$filterVal3)">1</xsl:when> <!-- found -->
									<xsl:otherwise></xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
						</xsl:when>
						<xsl:when test="$filterName4='Bugzilla' and $filterVal4!=''">
							<xsl:for-each select="cvs:revision">
								<xsl:choose>
									<xsl:when test="contains(cvs:bugzilla,$filterVal4)">1</xsl:when> <!-- found -->
									<xsl:otherwise></xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
						</xsl:when>
						<xsl:when test="$filterName5='Comments' and $filterVal5!=''">
							<xsl:for-each select="cvs:revision">
								<xsl:choose>
									<xsl:when test="contains(cvs:comments,$filterVal5)">1</xsl:when> <!-- found -->
									<xsl:otherwise></xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
						</xsl:when>
						<xsl:otherwise>1</xsl:otherwise> <!-- no matching filters enabled, show all -->
					</xsl:choose>
					</xsl:variable>

					<!-- for testing only -->
					<!-- <xsl:value-of select="$doRow" /> -->

					<xsl:if test="contains($doRow,'1')">
						<tr valign="top">
							<xsl:choose>
								<xsl:when test="(position() mod 2 = 1)">
									<xsl:attribute name="class">dark-row</xsl:attribute>
								</xsl:when>
								<xsl:otherwise>
									<xsl:attribute name="class">light-row</xsl:attribute>
								</xsl:otherwise>
							</xsl:choose>

							<td align="right"><a name="{position()}"><xsl:value-of select="concat(position(),'.')" /></a></td>

							<xsl:variable name="fileNoCVSRoot"><xsl:choose>
								<xsl:when test="starts-with(cvs:file,'/cvsroot/webtools')">
									<xsl:value-of select="substring-after(cvs:file,'/cvsroot/webtools')" />
								</xsl:when>
								<xsl:otherwise><xsl:value-of select="cvs:file" /></xsl:otherwise>
							</xsl:choose></xsl:variable>
							<xsl:variable name="file"><xsl:value-of select="cvs:file" /></xsl:variable>
							<xsl:variable name="projPath"><xsl:choose>
								<xsl:when test="contains(cvs:file,'jst.')">/viewcvs/indexwebtools.cgi/~checkout~</xsl:when>
								<xsl:when test="contains(cvs:file,'wst.')">/viewcvs/indexwebtools.cgi/~checkout~</xsl:when>
								<xsl:otherwise>/viewcvs/indexwebtools.cgi/~checkout~</xsl:otherwise>
							</xsl:choose></xsl:variable>
							<xsl:variable name="projDetailPath"><xsl:choose>
								<xsl:when test="contains(cvs:file,'jst.')">/viewcvs/indexwebtools.cgi</xsl:when>
								<xsl:when test="contains(cvs:file,'wst.')">/viewcvs/indexwebtools.cgi</xsl:when>
								<xsl:otherwise>/viewcvs/indexwebtools.cgi</xsl:otherwise>
							</xsl:choose></xsl:variable>
							<xsl:variable name="cvsPath"><xsl:value-of select="concat($cvsServer,$projPath,$fileNoCVSRoot)" /></xsl:variable>
							<xsl:variable name="cvsDetailPath"><xsl:value-of select="concat($cvsServer,$projDetailPath,$fileNoCVSRoot)" /></xsl:variable>

							<xsl:for-each select="document('whatsnew-cvs.substitutions.xml')/subs/sub">
								<xsl:variable name="proj"><xsl:value-of select="substring-before(path,concat('/',catg))" /></xsl:variable>
								<xsl:variable name="projNoCVSRoot"><xsl:choose>
									<xsl:when test="starts-with($proj,'/cvsroot/webtools')">
										<xsl:value-of select="substring-after($proj,'/cvsroot/webtools')" />
									</xsl:when>
									<xsl:otherwise><xsl:value-of select="$proj" /></xsl:otherwise>
								</xsl:choose></xsl:variable>
								<xsl:choose>
									<xsl:when test="starts-with($file,path)">
										<td><a target="_cvs" class="black-no-underline" href="{concat($cvsServer,$projPath,$projNoCVSRoot,'/',catg)}"><b><xsl:value-of select="catg" /></b></a></td>
										<td width="15"><a target="_cvs" class="black-no-underline" href="{concat($cvsServer,$projPath,$projNoCVSRoot,'/',catg)}"><img border="0" alt="CVS project folder" src="{$CVSPREWTP}/xml/icon-folder.gif"/></a></td>
									</xsl:when>
									<xsl:otherwise>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
									<!-- file links -->
									<td width="14"><a target="_cvs" class="black-no-underline" href="{$cvsDetailPath}"><img border="0" alt="CVS file details" src="{$CVSPREWTP}/xml/icon-file-detail.gif"/></a></td>
									<td width="13"><a target="_cvs" class="black-no-underline" href="{$cvsPath}"><img border="0" alt="CVS file source (or download)" src="{$CVSPREWTP}/xml/icon-file.gif"/></a></td>
									<td colspan="1"><a target="_cvs" class="black-no-underline" href="{$cvsPath}"><xsl:value-of select="translate(substring-after($fileNoCVSRoot,path),'/',' ')" /></a></td>
							<xsl:variable name="firstRev">
								<xsl:value-of select="cvs:revision/cvs:rev" />
							</xsl:variable>
							<xsl:variable name="lastRev">
								<xsl:for-each select="cvs:revision">
									<xsl:if test="position()=last()">
										<xsl:apply-templates select="cvs:rev"/> 
									</xsl:if>
								</xsl:for-each>
							</xsl:variable>
							<td><nobr>
								<xsl:choose>
									<xsl:when test="string($firstRev)=string($lastRev)">
										<i class="red-dark"><xsl:value-of select="$lastRev"/></i>
									</xsl:when>
									<xsl:otherwise>
										<a target="_cvs" class="red-no-underline" href="{$cvsPath}.diff?r1={$firstRev}&amp;r2={$lastRev}">
										<xsl:value-of select="$firstRev"/> 
										&gt; <xsl:value-of select="$lastRev" />
										</a> 
									</xsl:otherwise>
								</xsl:choose>
							(<span class="navy"><xsl:value-of select="cvs:branch"/></span>, 
							-<xsl:value-of select="cvs:keywords"/>)</nobr></td>
							<td width="100%" colspan="2">
								<table>
									<xsl:for-each select="cvs:revision">

										<!-- add filters -->
										<xsl:variable name="doSubRow">
										<xsl:choose>
											<xsl:when test="$filterName2='Date' and $filterVal2!=''">
												<xsl:if test="contains(cvs:date,$filterVal2)">2</xsl:if> <!-- found -->
											</xsl:when>
											<xsl:when test="$filterName3='Author' and $filterVal3!=''">
												<xsl:if test="contains(cvs:author,$filterVal3)">2</xsl:if> <!-- found -->
											</xsl:when>
											<xsl:when test="$filterName4='Bugzilla' and $filterVal4!=''">
												<xsl:if test="contains(cvs:bugzilla,$filterVal4)">2</xsl:if> <!-- found -->
											</xsl:when>
											<xsl:when test="$filterName5='Comments' and $filterVal5!=''">
												<xsl:if test="contains(cvs:comments,$filterVal5)">2</xsl:if> <!-- found -->
											</xsl:when>
											<xsl:when test="$filterVal2='' and $filterVal3='' and $filterVal4='' and $filterVal5=''">2</xsl:when> <!-- no filter -->
										</xsl:choose>
										</xsl:variable>

										<!-- for testing only -->
										<!-- <xsl:value-of select="$doSubRow" /> -->

										<xsl:if test="contains($doSubRow,'2')">
											<tr valign="top">
												<td><nobr><xsl:value-of select="cvs:date"/></nobr></td>
												<td>
												<xsl:variable name="prevRev">
													<xsl:apply-templates select="cvs:rev"/>
												</xsl:variable>
												<xsl:choose>
													<xsl:when test="string(rev)!='1.1'">
														<a target="_cvs" class="red-no-underline" href="{$cvsPath}.diff?r1={rev}&amp;r2={$prevRev}"><xsl:value-of select="rev"/></a>
													</xsl:when>
													<xsl:otherwise>
														<i class="red-dark"><xsl:value-of select="cvs:rev"/></i>
													</xsl:otherwise>
												</xsl:choose>
												</td>
												<td><xsl:value-of select="cvs:author"/></td>
												<xsl:choose>
													<xsl:when test="cvs:bugzilla">
														<td>[<a class="green-no-underline" href="https://bugs.eclipse.org/bugs/show_bug.cgi?id={cvs:bugzilla}" target="_bugz"><xsl:value-of select="cvs:bugzilla"/></a>]</td>
													</xsl:when>
													<xsl:otherwise>
														<td>  </td>
													</xsl:otherwise>
												</xsl:choose>
												<td>
													<xsl:value-of select="cvs:comments"/>
												</td>
											</tr>
										</xsl:if>
									</xsl:for-each>
								</table>
							</td>
						</tr>
					</xsl:if>
				</xsl:for-each>
			</table>
			</body>
			</html>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="cvs:rev">
		<xsl:if test="string(.)='1.1'">	<!-- simple case, 1.1 -->
			<xsl:value-of select="1.1" />
		</xsl:if>
		<xsl:if test="string(.)!='1.1'">
			<xsl:choose>
				<xsl:when test="contains(substring-after(substring-after(.,'.'),'.'),'.') and substring(.,string-length(.),1)!='1'"> 
					<!-- check if 1.6.2.2 : 1.6.2 . (2-1) : 1.6.2.1  -->
					<xsl:value-of select="concat(
						substring(.,1,string-length(.)-2),
						'.',
						number(substring(.,string-length(.)))-1
					)"/>
				</xsl:when>
				<xsl:when test="contains(substring-after(substring-after(.,'.'),'.'),'.') and substring(.,string-length(.),1)='1'"> 
					<!-- check if 1.6.2.1 : 1 . 6 : 1.6  -->
					<xsl:value-of select="concat(
						substring-before(.,'.'), 
						'.',
						substring-before(substring-after(.,'.'),'.')
					)"/>
				</xsl:when>
				<xsl:when test="contains(substring-after(.,'.'),'.') and substring(.,string-length(.),1)!='1'"> 
					<!-- check if 1.5.4 : 1.5 . (4-1) : 1.5.3 [is this a valid cvs range?] -->
					<xsl:value-of select="concat(
						substring(.,1,string-length(.)-2),
						'.',
						number(substring(.,string-length(.)))-1
					)"/>
				</xsl:when>
				<xsl:when test="contains(substring-after(.,'.'),'.') and substring(.,string-length(.),1)='1'"> 
					<!-- check if 1.5.1 : 1 . (5-1) : 1.4 [is this a valid cvs range?] -->
					<xsl:value-of select="concat(
						substring-before(.,'.'), 
						'.', 
						string(floor(number(substring-after(.,'.'))) - 1)
					)"/>
				</xsl:when>
				<xsl:otherwise>	
					<!-- check if 1.40 : 1 . (40-1) : 1.39 -->
					<xsl:value-of select="concat(
						substring-before(.,'.'),
						'.',
						number(substring-after(.,'.'))-1
					)"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:if>
	</xsl:template>

	<xsl:template match="colgroup">
		<td width="10"><b><xsl:value-of select="col/title" /></b></td>
	</xsl:template>
</xsl:stylesheet>
